<?php

namespace Drupal\Tests\inline_translation\Functional;

// phpcs:disable Drupal.Arrays.Array.LongLineDeclaration
use Drupal\Tests\content_translation\Functional\ContentTranslationUITestBase;
use Drupal\Tests\image\Kernel\ImageFieldCreationTrait;

/**
 * Tests the Inline Translation UI.
 *
 * @group inline_translation
 */
class InlineTranslationNodeUITest extends ContentTranslationUITestBase {
  use ImageFieldCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $entityTypeId = 'node';

  /**
   * {@inheritdoc}
   */
  protected $bundle = 'article';

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['node', 'inline_translation', 'block', 'field_ui', 'image'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setupBundle() {
    $this->drupalCreateContentType(['type' => 'article', 'name' => 'Article']);
    $this->createImageField('test_image', 'article', [], ['file_directory' => 'test-upload']);
    // TODO: Change the autogenerated stub.
    parent::setupBundle();
  }

  /**
   * Tests the inline translation UI.
   */
  public function testTranslationUi() {
    // Ensure the help message is shown even with prefixed paths.
    $this->drupalPlaceBlock('help_block', ['region' => 'content']);

    // Display the language selector.
    $this->drupalLogin($this->administrator);
    $edit = ['language_configuration[language_alterable]' => TRUE];
    $this->drupalPostForm('admin/structure/types/manage/article', $edit, 'Save content type');

    // Use inline translation.
    $this->drupalGet('admin/config/regional/content-language');
    $this->assertSession()->fieldExists('settings[node][article][settings][inline_translation]')->check();
    $this->assertSession()->buttonExists('Save configuration')->press();

    $this->drupalLogin($this->translator);
    $this->drupalGet('node/add/article');
    $this->assertSession()->fieldExists('title[0][value]')->setValue('English title');
    $this->assertSession()->fieldExists('title-fr[0][value]')->setValue('French title');
    $this->assertSession()->fieldExists('title-it[0][value]')->setValue('Italian title');
    $this->assertSession()->buttonExists('Save')->press();
    $this->assertSession()->pageTextContains('Article English title has been created.');
    $this->drupalGet('node/1/edit');
    $this->assertSession()->fieldValueEquals('title[0][value]', 'English title');
    $this->assertSession()->fieldValueEquals('title-fr[0][value]', 'French title');
    $this->assertSession()->fieldValueEquals('title-it[0][value]', 'Italian title');
    $this->assertSession()->fieldExists('title[0][value]')->setValue('English title 2');
    $this->assertSession()->fieldExists('title-fr[0][value]')->setValue('French title 2');
    $this->assertSession()->fieldExists('title-it[0][value]')->setValue('Italian title 2');
    $this->assertSession()->buttonExists('Save')->press();
    $this->assertSession()->pageTextContains('Article English title 2 has been updated.');
    $this->drupalGet('node/1/edit');
    $this->assertSession()->fieldValueEquals('title[0][value]', 'English title 2');
    $this->assertSession()->fieldValueEquals('title-fr[0][value]', 'French title 2');
    $this->assertSession()->fieldValueEquals('title-it[0][value]', 'Italian title 2');

    // Test that the configuration is removed on uninstall.
    $this->assertTrue(\Drupal::entityTypeManager()->getStorage('language_content_settings')->load($this->entityTypeId . '.' . $this->bundle)->getThirdPartySetting('inline_translation', 'enabled'));
    \Drupal::service('module_installer')->uninstall(['content_translation']);
    $this->assertEmpty(\Drupal::entityTypeManager()->getStorage('language_content_settings')->load($this->entityTypeId . '.' . $this->bundle)->getThirdPartySettings('inline_translation'));
  }

  /**
   * {@inheritdoc}
   */
  protected function getTranslatorPermissions() {
    return array_merge(parent::getTranslatorPermissions(), ['administer nodes', "edit any $this->bundle content", "create $this->bundle content"]);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditorPermissions() {
    return ['administer nodes', 'create article content'];
  }

  /**
   * {@inheritdoc}
   */
  protected function getAdministratorPermissions() {
    return array_merge(parent::getAdministratorPermissions(), ['access administration pages', 'administer content types', 'administer node fields', 'access content overview', 'bypass node access', 'administer languages', 'administer themes', 'view the administration theme']);
  }

}
